# /etc/cron.d/dsc-collector: crontab entries for the dsc-collector package

SHELL=/bin/sh
PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin

*/5 *    * * *   root    test -x /usr/share/dsc-statistics-collector/upload-prep && /usr/share/dsc-statistics-collector/upload-prep
